#!/usr/bin/perl
###############################################################################
#    Webmin Sysstats - internet-lib.pl
#
#    Webmin Sysstats Module
#    Copyright (C) 2001 by Eric Gerbier
#    Bug reports to: dbouius@users.sourceforge.net or gerbier@users.sourceforge.net
#    $Id: internet-lib.pl,v 0.3 2001/12/07 10:33:01 gerbier Exp $
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################

sub install_module {
  local ($minfo, $defaults) = ($_[0], $_[1]);

  local (%my_config);

  if (!-r $minfo->{'dbtables'} . "internet.rrd") {
    local ($cmd);

    $cmd = $config{'rrdtool'};
    $cmd .= " create " . $minfo->{'dbtables'} . "internet.rrd";
    $cmd .= " --start 0 --step 60";
    $cmd .= " DS:ftp:GAUGE:100:U:U";
    $cmd .= " DS:http:GAUGE:100:U:U";
    $cmd .= " RRA:AVERAGE:0.5:1:70";
    $cmd .= " RRA:AVERAGE:0.5:2:750";
    $cmd .= " RRA:AVERAGE:0.5:15:650";
    $cmd .= " RRA:AVERAGE:0.5:50:650";

    `$cmd`;
  }

  $my_config{'sample_rate'} = '1';

  $my_config{'internet'} = join(',',
			   $defaults->{'ftp_colour'},
			   $defaults->{'http_colour'}
			  );

  &write_config($minfo, \%my_config, $defaults);

  &install_script($minfo, 'internet.sh');

  return 1;
}

sub remove_module {
}

sub mod_check_runstop {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  return $my_config->{'runstop'};
}

sub mod_runstop {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  $my_config->{'runstop'} = $my_config->{'runstop'} ? '1' : '0';
  &write_config($minfo, $my_config, $defaults);
}

sub mod_check_upgrade {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  return 0;
}

sub mod_upgrade {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  local (%old_config);
}

sub graph_data {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  $rrdtool = $config{'rrdtool'};
  $sizex = $config{'graph_sizex'};
  $sizey = $config{'graph_sizey'};

  ($ftp_colour, $http_colour) = split(',', $my_config->{'internet'});

  $rrd_detals  = " -l 0 -v 'number of syslog lines'";
  $rrd_detals .= " DEF:ftp=" . $minfo->{'dbtables'} . "internet.rrd:ftp:AVERAGE";
  $rrd_detals .= " DEF:http=" . $minfo->{'dbtables'} . "internet.rrd:http:AVERAGE";
  $rrd_detals .= " COMMENT:'Type        Max      Min  Average\\n'";
  $rrd_detals .= " LINE1:ftp#" . $ftp_colour . ":'FTP'";
  $rrd_detals .= " GPRINT:ftp:MAX:'%6.2lf'";
  $rrd_detals .= " GPRINT:ftp:MIN:'%6.2lf'";
  $rrd_detals .= " GPRINT:ftp:AVERAGE:'%6.2lf\\n'";
  $rrd_detals .= " LINE2:http#" . $http_colour . ":'HTTP  '";
  $rrd_detals .= " GPRINT:http:MAX:'%6.2lf'";
  $rrd_detals .= " GPRINT:http:MIN:'%6.2lf'";
  $rrd_detals .= " GPRINT:http:AVERAGE:'%6.2lf\\n'";

  foreach $graph ('hour', 'day', 'week', 'month') {
    if ($graph eq 'hour') {
      $time = "-3600";
      $title = "'The last hour'";
    } elsif ($graph eq 'day') {
      $time = "-86400";
      $title = "'The last day'";
    } elsif ($graph eq 'week') {
      $time = "-604800";
      $title = "'The last week'";
    } elsif ($graph eq 'month') {
      $time = "-18748800";
      $title = "'The last month'";
    }

    $common  = $rrdtool . " graph " . $minfo->{'tmpgname'} . "_" . $graph . ".png";
    $common .= " --start " . $time . " --end -60";
    $common .= " -h " . $sizey . " -w " . $sizex;
    $common .= " -a PNG --title=" . $title;

    $sig = " COMMENT:'\\n' COMMENT:\"Created: `date`\"";

    system($common . $rrd_detals . $sig);
  }
}

sub mod_config {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  ($ftp_colour, $http_colour) = split(',', $my_config->{'internet'});
  print "<table width=100%><tr><td><b>";
  print  $text{'module_subtitle'};
  print "</b></td></tr></table>\n";

  print "<form action=save_config.cgi>\n";
  print  "<input type=hidden name=idx value='$in{'idx'}'>\n";

  print   "<table width=100%>\n";
  print    "<tr $cb>";
  print     "<td>";
  print      "<table width=100%>\n";

  print       "<tr>";
  print        "<td width=15%>";
  print         "<b>" . $text{'module_word_ftp'} . "</b>";
  print        "</td>";

  print        "<td width=35%>";
  &option_colourselect('ftp', $ftp_colour,
		       $defaults->{'ftp_colour'}
		      );
  print        "</td>";
  print        "<td width=15%>";
  print         "<b>" . $text{'module_word_http'} . "</b>";
  print        "</td>";

  print        "<td width=35%>";
  &option_colourselect('http', $http_colour,
		       $defaults->{'http_colour'}
		      );
  print        "</td>";
  print       "</tr>";
  print      "</table>";
  print     "</td>";
  print    "</tr>";
  print   "</table>\n";
  print  "<input type=submit value='" . $text{'word_save'} . "'>";
  print "</form>\n";
}

sub mod_config_save {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  $my_config->{'internet'} = join ( ',',
			     ($in{'ftp1_def'} eq 'default') ? $defaults->{'ftp_colour'}
			     : $in{'ftp2_def'} ? $in{'ftp2_def'} : $in{'ftp1_def'},
			     ($in{'http1_def'} eq 'default') ? $defaults->{'http_colour'}
			     : $in{'http2_def'} ? $in{'http2_def'} : $in{'http1_def'}
			   );

  &write_config($minfo, $my_config, $defaults);
}

sub write_config {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  local (%new_config, %sh_config);

  $new_config{'sample_rate'} = $my_config->{'sample_rate'};
  $sh_config{'sample_rate'} = $my_config->{'sample_rate'};
  $new_config{'runstop'} = $my_config->{'runstop'};
  $new_config{'internet'} = $my_config->{'internet'};

  if (!$new_config{'runstop'}) {
    $sh_config{'internet'} = 1;
  }

  unlink ($minfo->{'config_file'});
  unlink ($minfo->{'config_dir'} . 'internet.cfg');

  &write_file($minfo->{'config_file'}, \%new_config);
  &write_file($minfo->{'config_dir'} . 'internet.cfg', \%sh_config);
}

return 1;

